/********************************************************************************
** Form generated from reading UI file 'breezeconfigurationui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_BREEZECONFIGURATIONUI_H
#define UI_BREEZECONFIGURATIONUI_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "config/breezeexceptionlistwidget.h"
#include "kcolorbutton.h"

QT_BEGIN_NAMESPACE

class Ui_BreezeConfigurationUI
{
public:
    QVBoxLayout *verticalLayout;
    QTabWidget *tabWidget;
    QWidget *tab;
    QFormLayout *formLayout_3;
    QLabel *label_3;
    QComboBox *titleAlignment;
    QLabel *label_4;
    QComboBox *buttonSize;
    QCheckBox *outlineCloseButton;
    QCheckBox *drawBorderOnMaximizedWindows;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer_2;
    QLabel *drawBorderOnMaximizedWindowsHelpLabel;
    QCheckBox *drawBackgroundGradient;
    QCheckBox *roundedCorners;
    QWidget *tab_4;
    QFormLayout *formLayout;
    QLabel *label;
    QComboBox *shadowSize;
    QLabel *label_2;
    QSpinBox *shadowStrength;
    QLabel *label_5;
    KColorButton *shadowColor;
    QCheckBox *drawOutline;
    QWidget *tab_3;
    QVBoxLayout *verticalLayout_3;
    Breeze::ExceptionListWidget *exceptions;

    void setupUi(QWidget *BreezeConfigurationUI)
    {
        if (BreezeConfigurationUI->objectName().isEmpty())
            BreezeConfigurationUI->setObjectName("BreezeConfigurationUI");
        BreezeConfigurationUI->resize(469, 418);
        verticalLayout = new QVBoxLayout(BreezeConfigurationUI);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        tabWidget = new QTabWidget(BreezeConfigurationUI);
        tabWidget->setObjectName("tabWidget");
        tabWidget->setDocumentMode(true);
        tab = new QWidget();
        tab->setObjectName("tab");
        formLayout_3 = new QFormLayout(tab);
        formLayout_3->setObjectName("formLayout_3");
        formLayout_3->setFormAlignment(Qt::AlignmentFlag::AlignHCenter|Qt::AlignmentFlag::AlignTop);
        label_3 = new QLabel(tab);
        label_3->setObjectName("label_3");

        formLayout_3->setWidget(0, QFormLayout::ItemRole::LabelRole, label_3);

        titleAlignment = new QComboBox(tab);
        titleAlignment->addItem(QString());
        titleAlignment->addItem(QString());
        titleAlignment->addItem(QString());
        titleAlignment->addItem(QString());
        titleAlignment->setObjectName("titleAlignment");

        formLayout_3->setWidget(0, QFormLayout::ItemRole::FieldRole, titleAlignment);

        label_4 = new QLabel(tab);
        label_4->setObjectName("label_4");

        formLayout_3->setWidget(1, QFormLayout::ItemRole::LabelRole, label_4);

        buttonSize = new QComboBox(tab);
        buttonSize->addItem(QString());
        buttonSize->addItem(QString());
        buttonSize->addItem(QString());
        buttonSize->addItem(QString());
        buttonSize->addItem(QString());
        buttonSize->setObjectName("buttonSize");

        formLayout_3->setWidget(1, QFormLayout::ItemRole::FieldRole, buttonSize);

        outlineCloseButton = new QCheckBox(tab);
        outlineCloseButton->setObjectName("outlineCloseButton");

        formLayout_3->setWidget(2, QFormLayout::ItemRole::FieldRole, outlineCloseButton);

        drawBorderOnMaximizedWindows = new QCheckBox(tab);
        drawBorderOnMaximizedWindows->setObjectName("drawBorderOnMaximizedWindows");

        formLayout_3->setWidget(3, QFormLayout::ItemRole::FieldRole, drawBorderOnMaximizedWindows);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer_2 = new QSpacerItem(25, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);

        drawBorderOnMaximizedWindowsHelpLabel = new QLabel(tab);
        drawBorderOnMaximizedWindowsHelpLabel->setObjectName("drawBorderOnMaximizedWindowsHelpLabel");
        drawBorderOnMaximizedWindowsHelpLabel->setWordWrap(true);

        horizontalLayout->addWidget(drawBorderOnMaximizedWindowsHelpLabel);


        formLayout_3->setLayout(4, QFormLayout::ItemRole::FieldRole, horizontalLayout);

        drawBackgroundGradient = new QCheckBox(tab);
        drawBackgroundGradient->setObjectName("drawBackgroundGradient");

        formLayout_3->setWidget(5, QFormLayout::ItemRole::FieldRole, drawBackgroundGradient);

        roundedCorners = new QCheckBox(tab);
        roundedCorners->setObjectName("roundedCorners");

        formLayout_3->setWidget(6, QFormLayout::ItemRole::FieldRole, roundedCorners);

        tabWidget->addTab(tab, QString());
        tab_4 = new QWidget();
        tab_4->setObjectName("tab_4");
        formLayout = new QFormLayout(tab_4);
        formLayout->setObjectName("formLayout");
        formLayout->setFormAlignment(Qt::AlignmentFlag::AlignHCenter|Qt::AlignmentFlag::AlignTop);
        label = new QLabel(tab_4);
        label->setObjectName("label");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, label);

        shadowSize = new QComboBox(tab_4);
        shadowSize->addItem(QString());
        shadowSize->addItem(QString());
        shadowSize->addItem(QString());
        shadowSize->addItem(QString());
        shadowSize->addItem(QString());
        shadowSize->setObjectName("shadowSize");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, shadowSize);

        label_2 = new QLabel(tab_4);
        label_2->setObjectName("label_2");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, label_2);

        shadowStrength = new QSpinBox(tab_4);
        shadowStrength->setObjectName("shadowStrength");
        shadowStrength->setMinimum(10);
        shadowStrength->setMaximum(100);

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, shadowStrength);

        label_5 = new QLabel(tab_4);
        label_5->setObjectName("label_5");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, label_5);

        shadowColor = new KColorButton(tab_4);
        shadowColor->setObjectName("shadowColor");

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, shadowColor);

        drawOutline = new QCheckBox(tab_4);
        drawOutline->setObjectName("drawOutline");

        formLayout->setWidget(3, QFormLayout::ItemRole::FieldRole, drawOutline);

        tabWidget->addTab(tab_4, QString());
        tab_3 = new QWidget();
        tab_3->setObjectName("tab_3");
        verticalLayout_3 = new QVBoxLayout(tab_3);
        verticalLayout_3->setObjectName("verticalLayout_3");
        exceptions = new Breeze::ExceptionListWidget(tab_3);
        exceptions->setObjectName("exceptions");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(exceptions->sizePolicy().hasHeightForWidth());
        exceptions->setSizePolicy(sizePolicy);

        verticalLayout_3->addWidget(exceptions);

        tabWidget->addTab(tab_3, QString());

        verticalLayout->addWidget(tabWidget);

#if QT_CONFIG(shortcut)
        label_3->setBuddy(titleAlignment);
        label_4->setBuddy(buttonSize);
        label->setBuddy(shadowSize);
        label_2->setBuddy(shadowStrength);
#endif // QT_CONFIG(shortcut)

        retranslateUi(BreezeConfigurationUI);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(BreezeConfigurationUI);
    } // setupUi

    void retranslateUi(QWidget *BreezeConfigurationUI)
    {
        label_3->setText(tr2i18n("Tit&le alignment:", nullptr));
        titleAlignment->setItemText(0, tr2i18n("Left", nullptr));
        titleAlignment->setItemText(1, tr2i18n("Center", nullptr));
        titleAlignment->setItemText(2, tr2i18n("Center (Full Width)", nullptr));
        titleAlignment->setItemText(3, tr2i18n("Right", nullptr));

        label_4->setText(tr2i18n("B&utton size:", nullptr));
        buttonSize->setItemText(0, tr2i18n("Tiny", nullptr));
        buttonSize->setItemText(1, tr2i18n("Small", "@item:inlistbox Button size:"));
        buttonSize->setItemText(2, tr2i18n("Medium", "@item:inlistbox Button size:"));
        buttonSize->setItemText(3, tr2i18n("Large", "@item:inlistbox Button size:"));
        buttonSize->setItemText(4, tr2i18n("Very Large", "@item:inlistbox Button size:"));

        outlineCloseButton->setText(tr2i18n("Draw a circle around close button", nullptr));
        drawBorderOnMaximizedWindows->setText(tr2i18n("Draw border on maximized and tiled windows", nullptr));
        drawBorderOnMaximizedWindowsHelpLabel->setText(tr2i18n("This setting will allow you to resize from window edges touching screen edges. Scrollbars and window close buttons will shift positions and will not touch screen edges.", nullptr));
        drawBackgroundGradient->setText(tr2i18n("Draw titlebar background gradient", nullptr));
        roundedCorners->setText(tr2i18n("Round bottom corners of windows with no borders", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), tr2i18n("General", nullptr));
        label->setText(tr2i18n("Shadow size:", nullptr));
        shadowSize->setItemText(0, tr2i18n("None", "@item:inlistbox Button size:"));
        shadowSize->setItemText(1, tr2i18n("Small", "@item:inlistbox Button size:"));
        shadowSize->setItemText(2, tr2i18n("Medium", "@item:inlistbox Button size:"));
        shadowSize->setItemText(3, tr2i18n("Large", "@item:inlistbox Button size:"));
        shadowSize->setItemText(4, tr2i18n("Very Large", "@item:inlistbox Button size:"));

        label_2->setText(tr2i18n("Shadow strength:", "strength of the shadow (from transparent to opaque)"));
        shadowStrength->setSuffix(tr2i18n("%", nullptr));
        label_5->setText(tr2i18n("Shadow color:", nullptr));
        drawOutline->setText(tr2i18n("Draw window outline", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_4), tr2i18n("Shadows and Outline", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_3), tr2i18n("Window-Specific Overrides", nullptr));
        (void)BreezeConfigurationUI;
    } // retranslateUi

};

namespace Ui {
    class BreezeConfigurationUI: public Ui_BreezeConfigurationUI {};
} // namespace Ui

QT_END_NAMESPACE

#endif // BREEZECONFIGURATIONUI_H

